import {file_url, system_file, nodes} from "../../tool/nodes.js";

import {setView} from "../../route/setVue.js";

import alone from "../../tool/alone.js";

import path from "path";

export default async function (pathname, option = {}) {
    const self = {
        //配置
        option: Object.assign({
            //模式
            mode: process?.env?.NODE_ENV || 'development',
            //用户根目录 绝对路径
            pathname: process.cwd(),
            //开发目录相对路径
            path: (pathname || "src"),
            //配置文件名
            conf: "config/conf.js",
            //js文件名
            env: "config/env.js",
            //env.js开发key
            dev_key: "_dev_",
            //App.vue文件名
            app: "App.vue",
            //main.js文件名
            main: "main.js",
            //vite.js文件名
            vite: "config/vite.js",
            //index.html
            index: "index.html",
            //当前根目录 绝对路径
            dirname: file_url(import.meta.url, 3),
            //临时信息
            cache: "file/cache.json",
            //复制的目录 绝对路径
            copy_dir: alone.rtrim(file_url(import.meta.url, 3).replace(/\\/g, '/'), '/') + "/demo",
        }, option || {}),
        //判断是否/开头
        is_run(path) {
            return path.substring(0, 1) == '/';
        },
        //\替换成/
        rep(file) {
            return file.replace(/\\/g, '/');
        },
        trim_rep(file) {
            return alone.trim(this.rep(file), '/');
        },
        rtrim_rep(file) {
            return alone.rtrim(this.rep(file), '/');
        },
        //加载文件
        async import_file(file) {
            return await import(system_file(file));
        },
        //打包设置
        build: {
            //文件格式
            format: {
                img: ['jpeg', 'jpg', 'png', 'gif', 'svg', 'svgz', 'bmp', 'tiff', 'webp', 'heif', 'heic', 'ico'],
                font: ['ttf', 'otf', 'woff', 'woff2', 'eot'],
                audio: ['mp3', 'wav', 'ogg', 'aac', 'flac'],
                video: ['mp4', 'webm', 'ogv', 'avi', 'mov']
            },
            //文件格式 array
            format_arr: [],
            //打包配置
            config: {
                //格式 包含有'svg', 'svgz',生成vendor.svg, svg或者file体积太大时可以选择开启
                svg: ['svg', 'svgz'],
                //路径包含有/vue-,生成vendor.vue, chunk体积太大时可以选择开启
                vue: (file, format) => {
                    if (file.includes('/vue-') && format.includes('js')) {
                        return true;
                    }
                },
                //路径包含有/alone/,生成vendor.alone
                alone: (file, format) => {
                    if (file.includes('/alone-frame/') && format.includes('js')) {
                        return true;
                    }
                },
                //路径包含有/naive-ui,生成vendor.naive, 推荐开启
                naive: (file, format) => {
                    if (file.includes('/naive-ui') && format.includes('js')) {
                        return true;
                    }
                },
                //格式 包含图片,字体,音频,视频打包成 vendor.file, 推荐开启
                file: (file, format) => {
                    if (self.build.format_arr.includes(format)) {
                        return true;
                    }
                },
                //路径包含有node_modules helper 打包成vendor.chunk, 推荐开启
                chunk: (file, format) => {
                    const chunk = ['node_modules', 'helpers', '-helper'];
                    if (chunk.some(val => file.includes(val))) {
                        return true;
                    }
                }
            },
        },
        //jsconfig.json配置
        js_config(object, paths = {}) {
            paths["@root/*"] = ["./*"];
            paths["@public/*"] = ["./" + this.env.public + "/*", "./" + this.app_path + "/" + this.env.public + "/*"];
            paths["@view/*"] = ["./" + this.app_path + "/" + this.env.view + "/*"];
            paths["@src/*"] = ["./" + this.app_path + "/*"];
            object.compilerOptions = object.compilerOptions || {};
            object.compilerOptions.paths = Object.assign(object.compilerOptions.paths || {}, paths);
            //paths["@alone/*"] = [this.rtrim_rep(this.vite.get_path('.')) + "/*"];
            //object.include = ["./**/*", "./" + this.app_path + "/**/*"];
            return object;
        },
        //1
        handle_before() {
            //是否更新
            this.update = false;
            this.plugins = {};
            //文件格式 array
            this.build.format_arr = [].concat(...Object.values(this.build.format));
            //是否开发模式
            this.is_mode = this.option.mode == 'development';
            //当前根目录 node
            this.vite = nodes(this.option.dirname);
            //用户根目录 node
            this.node = nodes(this.option.pathname);
            //复制目录 node
            this.copy = nodes(this.option.copy_dir);
            //开发目录
            this.app_path = this.trim_rep(this.option.path);
            //开发目录 node
            this.app = nodes(this.node.get_path(this.app_path));
            this.option.conf = this.trim_rep(this.option.conf);
            this.option.env = this.trim_rep(this.option.env);
            this.option.app = this.trim_rep(this.option.app);
            this.option.main = this.trim_rep(this.option.main);
            this.option.vite = this.trim_rep(this.option.vite);
            this.option.cache = this.trim_rep(this.option.cache);
            this.option.index = this.trim_rep(this.option.index);
            return this;
        },
        //2
        async handle_env() {
            if (!this.app.is_exist(this.option.env)) {
                this.app.cp_file(this.copy.get_path("config/env.js"), this.option.env, false);
            }
            this.env = {};
            this.env_data = (await this.import_file(this.app.get_path(this.option.env))).default || {};
            Object.keys(this.env_data).forEach((k) => {
                if (this.is_mode) {
                    const key = this.option.dev_key + k;
                    if (this.env_data.hasOwnProperty(key)) {
                        this.env[k] = this.env_data[key];
                    } else if (k.substring(0, this.option.dev_key.length) == this.option.dev_key) {
                        this.env[k.substring(this.option.dev_key.length)] = this.env_data[k];
                    } else if (!this.env.hasOwnProperty(k)) {
                        this.env[k] = this.env_data[k];
                    }
                } else if (k.substring(0, this.option.dev_key.length) != this.option.dev_key) {
                    this.env[k] = this.env_data[k];
                }
            });
            this.app_ver = String.fromCharCode(Math.floor(Math.random() * 26) + 97) + alone.rand_str();
            this.env.mock = (this.is_mode && this.env.mock) ? this.app.get_path('mock') : "";
            this.env.public = this.trim_rep(this.env.public || 'public');
            this.env.view = this.trim_rep(this.env.view || 'view');
            this.env.app = this.env.app || this.app_ver;
            this.env.build = this.env.build || {};
            this.env.dts = this.env.dts || {};
            this.env.prefix = this.env.prefix || 'alone_';
            return this;
        },
        //3
        cache_file() {
            this.del_vite = false;
            this.app_info = this.app.dir_file_info(this.app.get_path('.')) || {};
            this.cache = this.vite.get_json(this.option.cache) || {info: {}};
            if (!alone.isset(this.cache) ||
                this.cache.path != this.app_path
                || this.cache.view != this.env.view
                || this.app_info?.birthtimeMs !== this.cache?.info?.birthtimeMs
            ) {
                this.del_vite = true;
                this.update = true;
                this.vite.save_json(this.option.cache, {
                    path: this.app_path,
                    view: this.env.view,
                    info: this.app_info,
                });
            }
            return this;
        },
        //4
        jsconfig() {
            this.is_jsconfig = this.node.is_exist('jsconfig.json');
            if (this.update || !this.is_jsconfig) {
                this.jsconfig_object = !this.is_jsconfig ? this.vite.get_json('jsconfig.json') : this.node.get_json('jsconfig.json');
                this.node.save_json('jsconfig.json', this.js_config(this.jsconfig_object));
            }
            return this;
        },
        //5
        tsconfig() {
            this.is_tsconfig = this.node.is_exist('tsconfig.json');
            if (!this.is_tsconfig) {
                this.node.save_json('tsconfig.json', {compilerOptions: {noImplicitAny: false}});
            }
            return this;
        },
        //6
        copy_app() {
            if (!this.app.is_exist(this.option.app)) {
                this.update = true;
                this.app.create_dir(this.env.public);
                this.app.create_dir(this.env.view);
                this.app.cp_file(this.copy.get_path("App.vue"), this.option.app, false);
                this.app.cp_file(this.copy.get_path("main.js"), this.option.main, false);
                this.app.cp_file(this.copy.get_path("config/vite.js"), this.option.vite, false);
                this.app.cp_file(this.copy.get_path("config/conf.js"), this.option.conf, false);
                this.app.cp_dir_file(this.copy.get_path("view"), this.env.view, [], false);
                this.app.cp_dir_file(this.copy.get_path("mock"), 'mock', [], false);
                this.app.cp_dir_file(this.copy.get_path("style"), 'style', [], false);
            }

            if (!this.node.is_exist(this.env.public + "/layui/layui.js")) {
                this.node.cp_dir_file(this.vite.get_path("style/layui"), this.env.public + "/layui", [], false);
                this.node.cp_file(this.vite.get_path("style/lay.js"), this.env.public + "/layui/lay.js", false);
            }

            if (this.app.is_exist(this.env.public)) {
                this.node.cp_dir_file(this.app.get_path(this.env.public), this.env.public, [], false);
            }

            this.index_html = `<!DOCTYPE html><html><head></head><body></body></html>`;
            if (!this.app.is_exist(this.option.index)) {
                this.app.save_file(this.option.index, this.index_html);
            }

            this.git_text = `logs
*.log
npm-debug.log*
yarn-debug.log*
yarn-error.log*
pnpm-debug.log*
lerna-debug.log*
*.local
.vscode
.idea
.DS_Store
*.suo
*.ntvs*
*.njsproj
*.sln
*.sw?
node_modules
dist
/public
package-lock.json
pnpm-lock.yaml
jsconfig.json
tsconfig.json
*.d.ts
`
            if (!this.node.is_exist(".gitignore")) {
                this.node.save_file(".gitignore", this.git_text);
            }
            return this;
        },
        //7
        handle_dist() {
            if (alone.isset(this.env.dist)) {
                this.out_dir = this.is_run(this.env.dist) ? this.env.dist : this.node.get_path(this.trim_rep(this.env.dist));
            } else {
                this.dist = "dist/" + (this.app_path.split('/').length > 1 ? (this.app_path.split('/').slice(1).join('/')) : this.app_path);
                this.out_dir = this.node.get_path(this.dist);
            }
            return this;
        },
        //8
        handle_favicon() {
            if (this.env.favicon) {
                if (!this.is_run(this.env.favicon)) {
                    this.env.favicon = alone.trim(alone.trim(this.env.favicon, '.'), '/');
                    this.env.favicon = this.app.is_exist(this.env.favicon) ? this.env.favicon : "";
                } else {
                    this.env.favicon = this.node.is_exist(this.env.public + "/" + alone.trim(alone.trim(this.env.favicon, '.'), '/')) ? this.env.favicon : "";
                }
            }
            return this;
        },
        //9
        async handle_boj() {
            this.env_obj = {...this.env};
            this.conf_obj = (this.app.is_exist(this.option.conf) ? ((await this.import_file(this.app.get_path(this.option.conf))).default) : {}) || {};
            this.del_env = ['host', 'port', 'prefix', 'build', 'zip', 'public', 'mock', 'favicon', 'view', 'dist', 'layui', 'exclude', 'dts'];
            this.del_env.forEach((k) => ((this.env_obj.hasOwnProperty(k)) && delete this.env_obj[k]));
            this.conf_vite = (this.app.is_exist(this.option.vite) ? ((await this.import_file(this.app.get_path(this.option.vite))).default) : null) || null;
            return this;
        },
        //10
        handle_chunk() {
            this.chunk = {svg: false, vue: false, alone: true, naive: true, file: true, chunk: true};
            Object.keys(this.build.config).forEach((k) => {
                if (this.env.build[k] === true) {
                    this.chunk[k] = this.build.config[k];
                } else if (alone.isset(this.env.build[k])) {
                    this.chunk[k] = this.env.build[k];
                } else if (!this.env.build.hasOwnProperty(k) && this.chunk[k] === true) {
                    this.chunk[k] = this.build.config[k];
                } else {
                    delete this.chunk[k];
                }
            });
            if (alone.isset(this.env.build.callback) && alone.is_object(this.env.build.callback)) {
                Object.keys(this.env.build.callback).forEach((key) => {
                    if (alone.isset(this.env.build.callback[key])) {
                        this.chunk[key] = this.env.build.callback[key];
                    }
                });
            }
            return this;
        },
        //11
        handle_html_tag() {
            this.html_tag = [
                {tag: 'meta', attrs: {charset: 'UTF-8'}, injectTo: 'head-prepend'},
                {
                    tag: 'meta', attrs: {name: 'viewport', content: 'width=device-width, initial-scale=1.0'},
                    injectTo: 'head-prepend',
                },
                {
                    tag: 'div',
                    attrs: {id: this.env.app},
                    injectTo: 'body',
                    children: `<style>body{margin:0}.sk-flow{width:50px;height:50px;display:flex;justify-content:space-between}.sk-flow-dot{width:25%;height:25%;background-color:#79bbff;border-radius:50%;animation:sk-flow 1.4s cubic-bezier(.455,.03,.515,.955)0s infinite both}.sk-flow-dot:first-child{animation-delay:-.3s}.sk-flow-dot:nth-child(2){animation-delay:-.15s}.sk-main{width:100vw;height:100vh;display:flex;align-items:center;justify-content:center}@keyframes sk-flow{0%,100%,80%{transform:scale(.3)}40%{transform:scale(1)}}</style><div class="sk-main"><div class="sk-flow"><div class="sk-flow-dot"></div><div class="sk-flow-dot"></div><div class="sk-flow-dot"></div></div></div>`,
                }
            ];
            if (this.env.favicon) {
                this.html_tag = this.html_tag.concat([{
                    tag: 'link',
                    attrs: {icon: 'favicon', rel: "icon", href: this.env.favicon},
                    injectTo: 'head',
                }]);
            }
            this.lay_def = {status: true, link: "/layui/css/layui.css", script: "/layui/layui.js"};
            this.env.layui = this.env.layui || this.lay_def;
            if (this.env.layui.status !== false) {
                this.html_tag = this.html_tag.concat([
                    {
                        tag: 'script',
                        attrs: {
                            type: 'module',
                            src: (this.env.layui.script || this.lay_def.script) + "?v=" + this.app_ver
                        },
                        injectTo: 'head-prepend',
                    },
                    {
                        tag: 'link',
                        attrs: {
                            rel: 'stylesheet',
                            href: (this.env.layui.link || this.lay_def.link) + "?v=" + this.app_ver
                        },
                        injectTo: 'head-prepend',
                    },
                    {
                        tag: 'script',
                        attrs: {type: 'module'},
                        injectTo: 'head-prepend',
                        children: `window.$ = layui.$;`,
                    },
                ]);
            }
        },
        //启动
        async start() {
            this.handle_before();
            await this.handle_env();
            this.cache_file();
            this.jsconfig();
            this.tsconfig();
            this.copy_app();
            this.handle_dist();
            this.handle_favicon();
            await this.handle_boj();
            this.handle_chunk();
            this.handle_html_tag();
            const oldRoute = await this.get_route();
            const oldKey = JSON.stringify(Object.keys(oldRoute));
            const newKey = JSON.stringify(this.vue_route());
            if (oldKey != newKey) {
                const mainCache = 'node_modules/.vite/deps/_metadata.json';
                if (this.node.is_exist(mainCache)) {
                    this.node.delete_file(mainCache);
                } else {
                    this.del_vite = true;
                }
            }
            if (!this.is_mode || this.del_vite) {
                this.node.deletes_file('node_modules/.vite');
            }
            return this;
        },
        async get_route() {
            return (await this.import_file(this.vite.get_path('route/vueList.js'))).default || {};
        },
        publicDir() {
            return this.node.get_path(this.env.public);
        },
        define() {
            return {'process.env': this.env_obj, 'process.config': this.conf_obj, 'process.plugins': this.plugins};
        },
        alias() {
            return Object.assign({
                '@root': this.node.get_path('.'),
                '@public': this.node.get_path(this.env.public),
                '@view': this.app.get_path(this.env.view),
                '@src': this.app.get_path('.'),
                '@alone': this.vite.get_path('.')
            }, ((this.node.get_path('.') == this.vite.get_path('.')) ? {"alone-frame": this.vite.get_path('.')} : {}));
        },
        preview() {
            return {host: this.env.host || '127.0.0.1', port: this.env.port || '8088'};
        },
        server() {
            return {
                proxy: {
                    '/api': {
                        target: this.env.api,
                        changeOrigin: true,
                        rewrite: (path) => path.replace(/^\/api/, '')
                    }
                }
            };
        },
        vue_route({dir, file} = {}) {
            return setView(this.app.get_path(this.env.view), {dir: dir, file: file, run: true});
        },
        plugin_build() {
            return {
                outDir: this.out_dir,
                target: 'esnext',//'esnext', es2015 es2019
                minify: 'terser',//'terser',esbuild
                cssCodeSplit: true, // 如果设置为false，整个项目中的所有 CSS 将被提取到一个 CSS 文件中
                reportCompressedSize: false,//  关闭生成map文件 可以达到缩小打包体积
                chunkSizeWarningLimit: 1500,
                sourcemap: false, // 这个生产环境一定要关闭，不然打包的产物会很大
                terserOptions: {
                    format: {
                        beautify: false, // 不格式化输出
                        comments: false, // 不保留注释
                    },
                    mangle: {
                        toplevel: true, // 混淆顶级变量和函数
                    },
                    compress: {
                        properties: true,
                        dead_code: true,    // 删除未使用的代码
                        drop_console: true,//打包时删除console
                        drop_debugger: true,//打包时删除 debugger
                    },
                    sourceMap: true, // 生成源映射文件
                    ecma: 2015,      // 使用 ECMAScript 2015 的特性
                    enclose: false,  // 不包裹代码在闭包中
                    keep_classnames: false, // 不保留类名
                    keep_fnames: false, // 不保留函数名
                    ie8: false,       // 不兼容 IE8
                    module: false,    // 不支持 ES6 模块语法
                    nameCache: null,  // 不使用名称缓存
                    safari10: false,  // 不兼容 Safari 10
                    toplevel: false,  // 不混淆顶级作用域的变量
                },
                rollupOptions: {
                    treeshake: true, // 开启 Tree Shaking，消除未使用的代码，减小最终的包大小
                    output: {
                        hashCharacters: 'hex',
                        chunkFileNames: (obj) => {
                            const name = (obj.name).toString().toLowerCase()
                            return 'assets/js/' + name + '.[hash].js';
                        },
                        entryFileNames: (obj) => {
                            const name = (obj.name).toString().toLowerCase()
                            return 'assets/js/' + name + '.[hash].js';
                        },
                        assetFileNames: (obj) => {
                            const type = obj.type.toString().toLowerCase();
                            if (type === 'asset' && (obj.name || obj.names)) {
                                const file = (obj.name || (obj.names.slice(0, 1).join(''))).toString();
                                const files = file.replace(/\\/g, '/').toLowerCase();
                                const format = path.extname(files).slice(1);
                                const array = Object.keys(this.build.format);
                                for (const key of array) {
                                    const val = this.build.format[key];
                                    if (val && ((typeof val == 'function') ? val(files, format) : (Array.isArray(val) ? val.includes(format) : files.includes(val)))) {
                                        return 'assets/' + key + '/[name].[hash].[ext]';
                                    }
                                }
                            }
                            return 'assets/[ext]/[name].[hash].[ext]';
                        },
                        manualChunks: (file) => {
                            const files = file.replace(/\\/g, '/').toLowerCase();
                            const format = path.extname(files).slice(1);
                            const array = Object.keys(this.chunk);
                            for (const key of array) {
                                const val = this.chunk[key];
                                if (val && ((typeof val == 'function') ? val(files, format) : (Array.isArray(val) ? val.includes(format) : files.includes(val)))) {
                                    return "vendor." + key;
                                }
                            }
                        }
                    }
                }
            };
        }
    };
    return (await self.start());
};